==> netacct-mysql-0.73rc2.tar.gz: netacct-mysql-0.73rc2/src/capture.c <==
/* 
 * Network accounting
 * capture.c - capture raw packets - pcap version
 * (C) 1996 Ulrich Callmeier
 * rewritten experimental pcap interface to netacct
 * to work with linux, bsd ... not tested :(
 * geroy
 */

#include "netacct.h"

#ifdef HAVE_LIBPCAP
#include <pcap.h>
#endif

#ifndef _OpenBSD_
#include <net/ethernet.h>
#endif

#if defined(_FreeBSD_) || defined(_OpenBSD_)
#include <machine/param.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#endif

#ifdef _FreeBSD_
#include <netinet/ip_compat.h>
#endif

#ifdef _OpenBSD_
#include <netinet/if_ether.h>
#endif

#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>


#include <stdio.h>
#include <stdlib.h>
/*#include <netdb.h>*/
#include <string.h>


void handle_ip(unsigned char buf[], char *devname, char *user);

pcap_t *pcap;
char errbuff[PCAP_ERRBUF_SIZE];

#define PCAP_SNAPLEN 128
#define PCAP_TMOUT 1000

void handle_frame (unsigned char[], int);

void init_capture()
{
  struct promisc_device *p;

  pcap = NULL;

  p = cfg -> promisc;
    
  if(p!=NULL)
    {
      pcap = pcap_open_live(p -> name, PCAP_SNAPLEN, 1, PCAP_TMOUT, errbuff);
      
      if(pcap == NULL)
	{
	  syslog(LOG_ERR, "can't pcap_open_live: %s\n", errbuff);
	  daemon_stop(0);
	}
    }
  
  DEBUG(DBG_MISC, sprintf(dbg, "%s set to promiscous mode\n", p -> name));
  
}

void exit_capture(void)
{
  if(pcap != NULL) pcap_close (pcap);
  pcap = NULL;
}

inline int onnet(unsigned long int addr, struct ipnetwork *net)
{
  return ((addr & net -> netmask) == net -> netnumber);
}

int onnetlist(unsigned long int addr, struct ipnetwork *netlist)
{
    while(netlist!=NULL)
	{
	    if(onnet(addr, netlist))
		{
		    return 1;
		}
	    netlist = netlist -> next;
	}
    return 0;
}

void do_packet(u_char *usr, const struct pcap_pkthdr *h, const u_char *p)
{
  handle_frame(p, h->len);
}

void packet_loop()
{
  int res;
  while(running)
    {
      res = pcap_dispatch(pcap, -1, do_packet, NULL); 
      if(res == -1) 
	{
	  syslog(LOG_ERR, "pcap_dispatch: %s\n", pcap_geterr(pcap));
	  DEBUG(DBG_ERR, sprintf(dbg,"pcap_dispatch: %s\n", pcap_geterr(pcap)));
	}
    }
}

/* by geroy -- not used??!? */
/* FILE *acct_file = stderr; */

void
handle_frame (unsigned char buf[], int length)
{
#if defined(_FreeBSD_) || defined(_OpenBSD_) 
  static struct ip tmp_iphdr;
#else
  static struct iphdr tmp_iphdr;
#endif
  unsigned short srcport, dstport;
  struct tcphdr tmp_tcphdr;
  struct udphdr tmp_udphdr;
  struct icmp tmp_icmphdr;
  int found = 0;
  struct mon_host_struct *ptr;

  if(buf[12] * 256 + buf[13] == ETHERTYPE_IP)
    {
      memcpy (&tmp_iphdr, &(buf[14]), sizeof (tmp_iphdr));
      

      found = 0;
      if(cfg->hostlist) { /* we specified at least one hostlimit tag */
        /* if we don't monitor this IP, bail now - mk */
          for(ptr=cfg->hostlist;ptr && !found;ptr=ptr->next)
	  {

#if defined(_FreeBSD_) || defined(_OpenBSD_)	  
		if(ptr->ipaddr == tmp_iphdr.ip_src.s_addr
	 	|| ptr->ipaddr == tmp_iphdr.ip_dst.s_addr)
#else
                if(ptr->ipaddr == tmp_iphdr.saddr
		|| ptr->ipaddr == tmp_iphdr.daddr)
#endif
                    found = 1;
          
          if(!found) {
                packets->ignored++;
                continue;
             }
          }
       }
#if defined(_FreeBSD_) || defined(_OpenBSD_)
      if((tmp_iphdr.ip_src.s_addr & cfg->ignoremask) ==
	 (tmp_iphdr.ip_dst.s_addr & cfg->ignoremask))
#else
      if((tmp_iphdr.saddr & cfg->ignoremask) ==
	 (tmp_iphdr.daddr & cfg->ignoremask))
#endif	      
        {
          packets->local++;
          return;
        }
      else
	{

#if defined(_FreeBSD_) || defined(_OpenBSD_)
	if(onnetlist(tmp_iphdr.ip_src.s_addr,cfg->ignorenet) ||
	     onnetlist(tmp_iphdr.ip_dst.s_addr, cfg->ignorenet))
#else
	if(onnetlist(tmp_iphdr.saddr,cfg->ignorenet) ||
             onnetlist(tmp_iphdr.daddr, cfg->ignorenet))
#endif		
		
            {
#if defined(_FreeBSD_) || defined(_OpenBSD_)		    
              if(!(onnetlist(tmp_iphdr.ip_src.s_addr,cfg->dontignore) ||
		   onnetlist(tmp_iphdr.ip_dst.s_addr, cfg->dontignore)))
#else
              if(!(onnetlist(tmp_iphdr.saddr,cfg->dontignore) ||
                   onnetlist(tmp_iphdr.daddr, cfg->dontignore)))
#endif		      
					    
                {
                  if(debug_level & DBG_IGNORE)
                    {
                      char tmp[18];
#if defined(_FreeBSD_) || defined(_OpenBSD_)		      
                      strcpy(tmp, intoa(tmp_iphdr.ip_src.s_addr));
#else
		      strcpy(tmp, intoa(tmp_iphdr.saddr));
#endif		      

#if defined(_FreeBSD_) || defined(_OpenBSD_)		      
                      DEBUG(DBG_IGNORE, sprintf(dbg, "netignored: %s -> %s\n",
                            tmp,intoa(tmp_iphdr.ip_dst.s_addr)));
#else
                      DEBUG(DBG_IGNORE, sprintf(dbg, "netignored: %s -> %s\n",
		            tmp,intoa(tmp_iphdr.daddr)));
#endif		      
		      
                    }
                  packets->netignored++;
                  return;
                }
            }
	  packets->ip++;
	  
#if defined(_FreeBSD_) || defined(_OpenBSD_)	  
	  switch(tmp_iphdr.ip_p)
#else
          switch(tmp_iphdr.protocol)
#endif	
	    {  
	    case IPPROTO_UDP:
	      packets->ip_udp++;
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      memcpy (&tmp_udphdr, &buf[14 + tmp_iphdr.ip_hl * 4], sizeof (tmp_udphdr));
#else	      
	      memcpy (&tmp_udphdr, &buf[14 + tmp_iphdr.ihl * 4], sizeof (tmp_udphdr));
#endif	      

#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      srcport = tmp_udphdr.uh_sport;
	      dstport = tmp_udphdr.uh_dport;
#else	      
	      srcport = tmp_udphdr.source;
	      dstport = tmp_udphdr.dest;
#endif
	      break;
	    case IPPROTO_TCP:
	      packets->ip_tcp++;
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      memcpy (&tmp_tcphdr, &buf[14 + tmp_iphdr.ip_hl * 4], sizeof (tmp_tcphdr));
#else
	      memcpy (&tmp_tcphdr, &buf[14 + tmp_iphdr.ihl * 4], sizeof (tmp_tcphdr));
#endif	     

#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      srcport = tmp_tcphdr.th_sport;
              dstport = tmp_tcphdr.th_dport;
#else	      	      
	      srcport = tmp_tcphdr.source;
	      dstport = tmp_tcphdr.dest;
#endif
	      break;
	    case IPPROTO_ICMP:
	      packets->ip_icmp++;
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      memcpy (&tmp_icmphdr, &buf[14 + tmp_iphdr.ip_hl * 4], sizeof (tmp_icmphdr));
#else
	      memcpy (&tmp_icmphdr, &buf[14 + tmp_iphdr.ihl * 4], sizeof (tmp_icmphdr));
#endif	      
	      srcport = tmp_icmphdr.icmp_type;
	      dstport = tmp_icmphdr.icmp_code;

	      break;
	    default:
	      packets->ip_other++;
	      srcport = dstport = 0;
	      break;
	    }
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	  register_packet(tmp_iphdr.ip_src.s_addr,tmp_iphdr.ip_dst.s_addr,
			  tmp_iphdr.ip_p, srcport, dstport, 
			  ntohs(tmp_iphdr.ip_len), strdup(cfg->promisc->name),
			  NULL);
#else

	  register_packet(tmp_iphdr.saddr,tmp_iphdr.daddr,
			  tmp_iphdr.protocol, srcport, dstport, 
			  ntohs(tmp_iphdr.tot_len), strdup(cfg->promisc->name),
			  NULL);

#endif
	}
    }
  else
    {
      /* ETH_P_ARP, ETH_P_RARP, ETH_P_IPX, etc. */
      packets -> ignored ++;
    }
}

==> netacct-mysql-0.74rc1.tar.gz: netacct-mysql-0.74rc1/src/capture.c <==
/* 
 * Network accounting
 * capture.c - capture raw packets - pcap version
 * (C) 1996 Ulrich Callmeier
 * rewritten experimental pcap interface to netacct
 * to work with linux, bsd ... 
 * geroy
 * added missing ntohs() to srcport, dstport
 */

#include "netacct.h"

#ifdef HAVE_LIBPCAP
#include <pcap.h>
#endif

#ifndef _OpenBSD_
#include <net/ethernet.h>
#endif

#if defined(_FreeBSD_) || defined(_OpenBSD_)
#include <machine/param.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#endif

#ifdef _FreeBSD_
#include <netinet/ip_compat.h>
#endif

#ifdef _OpenBSD_
#include <netinet/if_ether.h>
#endif

#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>


#include <stdio.h>
#include <stdlib.h>
/*#include <netdb.h>*/
#include <string.h>

char *rcs_revision_capture_c = "$Revision: 1.13 $";

void handle_ip(unsigned char buf[], char *devname, char *user);

pcap_t *pcap;
char errbuff[PCAP_ERRBUF_SIZE];

#define PCAP_SNAPLEN 128
#define PCAP_TMOUT 1000

void handle_frame (unsigned char[], int);

void init_capture()
{
  struct promisc_device *p;

  pcap = NULL;

  p = cfg -> promisc;
    
  if(p!=NULL)
    {
      pcap = pcap_open_live(p -> name, PCAP_SNAPLEN, 1, PCAP_TMOUT, errbuff);
      
      if(pcap == NULL)
	{
	  syslog(LOG_ERR, "can't pcap_open_live: %s\n", errbuff);
	  daemon_stop(0);
	}
    }
  
  DEBUG(DBG_MISC, sprintf(dbg, "%s set to promiscous mode\n", p -> name));
  
}

void exit_capture(void)
{
  if(pcap != NULL) pcap_close (pcap);
  pcap = NULL;
}

inline int onnet(unsigned long int addr, struct ipnetwork *net)
{
  return ((addr & net -> netmask) == net -> netnumber);
}

int onnetlist(unsigned long int addr, struct ipnetwork *netlist)
{
    while(netlist!=NULL)
	{
	    if(onnet(addr, netlist))
		{
		    return 1;
		}
	    netlist = netlist -> next;
	}
    return 0;
}

void do_packet(u_char *usr, const struct pcap_pkthdr *h, const u_char *p)
{
  handle_frame((char *)p, h->len); /* help by efimov :)*/
}

void packet_loop()
{
  int res;
  while(running)
    {
      res = pcap_dispatch(pcap, -1, do_packet, NULL); 
      if(res == -1) 
	{
	  syslog(LOG_ERR, "pcap_dispatch: %s\n", pcap_geterr(pcap));
	  DEBUG(DBG_ERR, sprintf(dbg,"pcap_dispatch: %s\n", pcap_geterr(pcap)));
	}
    }
}

/* by geroy -- not used??!? */
/* FILE *acct_file = stderr; */

void
handle_frame (unsigned char buf[], int length)
{
#if defined(_FreeBSD_) || defined(_OpenBSD_) 
  static struct ip tmp_iphdr;
#else
  static struct iphdr tmp_iphdr;
#endif
  unsigned short srcport, dstport;
  struct tcphdr tmp_tcphdr;
  struct udphdr tmp_udphdr;
  struct icmp tmp_icmphdr;
  int found = 0;
  struct mon_host_struct *ptr;

  if(buf[12] * 256 + buf[13] == ETHERTYPE_IP)
    {
      memcpy (&tmp_iphdr, &(buf[14]), sizeof (tmp_iphdr));
      

      found = 0;
      if(cfg->hostlist) { /* we specified at least one hostlimit tag */
        /* if we don't monitor this IP, bail now - mk */
          for(ptr=cfg->hostlist;ptr && !found;ptr=ptr->next)
	  {

#if defined(_FreeBSD_) || defined(_OpenBSD_)	  
		if(ptr->ipaddr == tmp_iphdr.ip_src.s_addr
	 	|| ptr->ipaddr == tmp_iphdr.ip_dst.s_addr)
#else
                if(ptr->ipaddr == tmp_iphdr.saddr
		|| ptr->ipaddr == tmp_iphdr.daddr)
#endif
                    found = 1;
          
          if(!found) {
                packets->ignored++;
                continue;
             }
          }
       }
#if defined(_FreeBSD_) || defined(_OpenBSD_)
      if((tmp_iphdr.ip_src.s_addr & cfg->ignoremask) ==
	 (tmp_iphdr.ip_dst.s_addr & cfg->ignoremask))
#else
      if((tmp_iphdr.saddr & cfg->ignoremask) ==
	 (tmp_iphdr.daddr & cfg->ignoremask))
#endif	      
        {
          packets->local++;
          return;
        }
      else
	{

#if defined(_FreeBSD_) || defined(_OpenBSD_)
	if(onnetlist(tmp_iphdr.ip_src.s_addr,cfg->ignorenet) ||
	     onnetlist(tmp_iphdr.ip_dst.s_addr, cfg->ignorenet))
#else
	if(onnetlist(tmp_iphdr.saddr,cfg->ignorenet) ||
             onnetlist(tmp_iphdr.daddr, cfg->ignorenet))
#endif		
		
            {
#if defined(_FreeBSD_) || defined(_OpenBSD_)		    
              if(!(onnetlist(tmp_iphdr.ip_src.s_addr,cfg->dontignore) ||
		   onnetlist(tmp_iphdr.ip_dst.s_addr, cfg->dontignore)))
#else
              if(!(onnetlist(tmp_iphdr.saddr,cfg->dontignore) ||
                   onnetlist(tmp_iphdr.daddr, cfg->dontignore)))
#endif		      
					    
                {
                  if(debug_level & DBG_IGNORE)
                    {
                      char tmp[18];
#if defined(_FreeBSD_) || defined(_OpenBSD_)		      
                      strcpy(tmp, intoa(tmp_iphdr.ip_src.s_addr));
#else
		      strcpy(tmp, intoa(tmp_iphdr.saddr));
#endif		      

#if defined(_FreeBSD_) || defined(_OpenBSD_)		      
                      DEBUG(DBG_IGNORE, sprintf(dbg, "netignored: %s -> %s\n",
                            tmp,intoa(tmp_iphdr.ip_dst.s_addr)));
#else
                      DEBUG(DBG_IGNORE, sprintf(dbg, "netignored: %s -> %s\n",
		            tmp,intoa(tmp_iphdr.daddr)));
#endif		      
		      
                    }
                  packets->netignored++;
                  return;
                }
            }
	  packets->ip++;
	  
#if defined(_FreeBSD_) || defined(_OpenBSD_)	  
	  switch(tmp_iphdr.ip_p)
#else
          switch(tmp_iphdr.protocol)
#endif	
	    {  
	    case IPPROTO_UDP:
	      packets->ip_udp++;
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      memcpy (&tmp_udphdr, &buf[14 + tmp_iphdr.ip_hl * 4], sizeof (tmp_udphdr));
#else	      
	      memcpy (&tmp_udphdr, &buf[14 + tmp_iphdr.ihl * 4], sizeof (tmp_udphdr));
#endif	      

#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      srcport = ntohs(tmp_udphdr.uh_sport);
	      dstport = ntohs(tmp_udphdr.uh_dport);
#else	      
	      srcport = ntohs(tmp_udphdr.source);
	      dstport = ntohs(tmp_udphdr.dest);
#endif
	      break;
	    case IPPROTO_TCP:
	      packets->ip_tcp++;
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      memcpy (&tmp_tcphdr, &buf[14 + tmp_iphdr.ip_hl * 4], sizeof (tmp_tcphdr));
#else
	      memcpy (&tmp_tcphdr, &buf[14 + tmp_iphdr.ihl * 4], sizeof (tmp_tcphdr));
#endif	     

#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      srcport = ntohs(tmp_tcphdr.th_sport);
              dstport = ntohs(tmp_tcphdr.th_dport);
#else	      	      
	      srcport = ntohs(tmp_tcphdr.source);
	      dstport = ntohs(tmp_tcphdr.dest);
#endif
	      break;
	    case IPPROTO_ICMP:
	      packets->ip_icmp++;
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	      memcpy (&tmp_icmphdr, &buf[14 + tmp_iphdr.ip_hl * 4], sizeof (tmp_icmphdr));
#else
	      memcpy (&tmp_icmphdr, &buf[14 + tmp_iphdr.ihl * 4], sizeof (tmp_icmphdr));
#endif	      
	      srcport = tmp_icmphdr.icmp_type;
	      dstport = tmp_icmphdr.icmp_code;

	      break;
	    default:
	      packets->ip_other++;
	      srcport = dstport = 0;
	      break;
	    }
#if defined(_FreeBSD_) || defined(_OpenBSD_)	      
	  register_packet(tmp_iphdr.ip_src.s_addr,tmp_iphdr.ip_dst.s_addr,
			  tmp_iphdr.ip_p, srcport, dstport, 
			  ntohs(tmp_iphdr.ip_len), cfg->promisc->name,
			  NULL);
#else

	  register_packet(tmp_iphdr.saddr,tmp_iphdr.daddr,
			  tmp_iphdr.protocol, srcport, dstport, 
			  ntohs(tmp_iphdr.tot_len), cfg->promisc->name,
			  NULL);

#endif
	}
    }
  else
    {
      /* ETH_P_ARP, ETH_P_RARP, ETH_P_IPX, etc. */
      packets -> ignored ++;
    }
}
